# Copyright 2018 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import enum


class ProtoType(enum.IntEnum):
    """The set of basic types in protocol buffers."""
    # These values come from google/protobuf/descriptor.proto
    DOUBLE = 1
    FLOAT = 2
    INT64 = 3
    UINT64 = 4
    INT32 = 5
    FIXED64 = 6
    FIXED32 = 7
    BOOL = 8
    STRING = 9
    MESSAGE = 11
    BYTES = 12
    UINT32 = 13
    ENUM = 14
    SFIXED32 = 15
    SFIXED64 = 16
    SINT32 = 17
    SINT64 = 18


# The default values for each type.
def get_default_value(proto_type):
    return {
        ProtoType.DOUBLE: 0.0,
        ProtoType.FLOAT: 0.0,
        ProtoType.INT64: 0,
        ProtoType.UINT64: 0,
        ProtoType.INT32: 0,
        ProtoType.FIXED64: 0,
        ProtoType.FIXED32: 0,
        ProtoType.BOOL: False,
        ProtoType.STRING: '',
        ProtoType.MESSAGE: None,
        ProtoType.BYTES: b'',
        ProtoType.UINT32: 0,
        ProtoType.ENUM: 0,
        ProtoType.SFIXED32: 0,
        ProtoType.SFIXED64: 0,
        ProtoType.SINT32: 0,
        ProtoType.SINT64: 0,
    }[proto_type]
